# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/TimeToLiveApi.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ai/h2o/featurestore/api/v1/TimeToLiveApi.proto',
  package='ai.h2o.featurestore.api.v1',
  syntax='proto3',
  serialized_options=b'B\022TimeToLiveApiProto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n.ai/h2o/featurestore/api/v1/TimeToLiveApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"L\n\x06Online\"B\n\x12TimeToLiveInterval\x12\x08\n\x04\x44\x41YS\x10\x00\x12\n\n\x06MONTHS\x10\x01\x12\t\n\x05HOURS\x10\x02\x12\x0b\n\x07MINUTES\x10\x03\"K\n\x07Offline\"@\n\x12TimeToLiveInterval\x12\t\n\x05YEARS\x10\x00\x12\x08\n\x04\x44\x41YS\x10\x01\x12\n\n\x06MONTHS\x10\x02\x12\t\n\x05HOURS\x10\x03\"\xdf\x01\n\nTimeToLive\x12\x13\n\x0bttl_offline\x18\x01 \x01(\r\x12T\n\x14ttl_offline_interval\x18\x02 \x01(\x0e\x32\x36.ai.h2o.featurestore.api.v1.Offline.TimeToLiveInterval\x12\x12\n\nttl_online\x18\x03 \x01(\r\x12R\n\x13ttl_online_interval\x18\x04 \x01(\x0e\x32\x35.ai.h2o.featurestore.api.v1.Online.TimeToLiveIntervalB\x14\x42\x12TimeToLiveApiProtob\x06proto3'
)



_ONLINE_TIMETOLIVEINTERVAL = _descriptor.EnumDescriptor(
  name='TimeToLiveInterval',
  full_name='ai.h2o.featurestore.api.v1.Online.TimeToLiveInterval',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DAYS', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MONTHS', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HOURS', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MINUTES', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=88,
  serialized_end=154,
)
_sym_db.RegisterEnumDescriptor(_ONLINE_TIMETOLIVEINTERVAL)

_OFFLINE_TIMETOLIVEINTERVAL = _descriptor.EnumDescriptor(
  name='TimeToLiveInterval',
  full_name='ai.h2o.featurestore.api.v1.Offline.TimeToLiveInterval',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='YEARS', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DAYS', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MONTHS', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HOURS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=167,
  serialized_end=231,
)
_sym_db.RegisterEnumDescriptor(_OFFLINE_TIMETOLIVEINTERVAL)


_ONLINE = _descriptor.Descriptor(
  name='Online',
  full_name='ai.h2o.featurestore.api.v1.Online',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ONLINE_TIMETOLIVEINTERVAL,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=78,
  serialized_end=154,
)


_OFFLINE = _descriptor.Descriptor(
  name='Offline',
  full_name='ai.h2o.featurestore.api.v1.Offline',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OFFLINE_TIMETOLIVEINTERVAL,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=156,
  serialized_end=231,
)


_TIMETOLIVE = _descriptor.Descriptor(
  name='TimeToLive',
  full_name='ai.h2o.featurestore.api.v1.TimeToLive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ttl_offline', full_name='ai.h2o.featurestore.api.v1.TimeToLive.ttl_offline', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ttl_offline_interval', full_name='ai.h2o.featurestore.api.v1.TimeToLive.ttl_offline_interval', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ttl_online', full_name='ai.h2o.featurestore.api.v1.TimeToLive.ttl_online', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ttl_online_interval', full_name='ai.h2o.featurestore.api.v1.TimeToLive.ttl_online_interval', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=234,
  serialized_end=457,
)

_ONLINE_TIMETOLIVEINTERVAL.containing_type = _ONLINE
_OFFLINE_TIMETOLIVEINTERVAL.containing_type = _OFFLINE
_TIMETOLIVE.fields_by_name['ttl_offline_interval'].enum_type = _OFFLINE_TIMETOLIVEINTERVAL
_TIMETOLIVE.fields_by_name['ttl_online_interval'].enum_type = _ONLINE_TIMETOLIVEINTERVAL
DESCRIPTOR.message_types_by_name['Online'] = _ONLINE
DESCRIPTOR.message_types_by_name['Offline'] = _OFFLINE
DESCRIPTOR.message_types_by_name['TimeToLive'] = _TIMETOLIVE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Online = _reflection.GeneratedProtocolMessageType('Online', (_message.Message,), {
  'DESCRIPTOR' : _ONLINE,
  '__module__' : 'ai.h2o.featurestore.api.v1.TimeToLiveApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.Online)
  })
_sym_db.RegisterMessage(Online)

Offline = _reflection.GeneratedProtocolMessageType('Offline', (_message.Message,), {
  'DESCRIPTOR' : _OFFLINE,
  '__module__' : 'ai.h2o.featurestore.api.v1.TimeToLiveApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.Offline)
  })
_sym_db.RegisterMessage(Offline)

TimeToLive = _reflection.GeneratedProtocolMessageType('TimeToLive', (_message.Message,), {
  'DESCRIPTOR' : _TIMETOLIVE,
  '__module__' : 'ai.h2o.featurestore.api.v1.TimeToLiveApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.TimeToLive)
  })
_sym_db.RegisterMessage(TimeToLive)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
