# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/RecommendationProtoApi.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ai/h2o/featurestore/api/v1/RecommendationProtoApi.proto',
  package='ai.h2o.featurestore.api.v1',
  syntax='proto3',
  serialized_options=b'B\033RecommendationProtoApiProto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n7ai/h2o/featurestore/api/v1/RecommendationProtoApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"L\n!RecommendationRegexMatchingPolicy\x12\r\n\x05regex\x18\x01 \x01(\t\x12\x18\n\x10percentage_match\x18\x02 \x01(\r\"\xbf\x01\n\"RecommendationSampleMatchingPolicy\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12!\n\x19\x66\x65\x61ture_set_major_version\x18\x02 \x01(\r\x12\x13\n\x0b\x63olumn_name\x18\x03 \x01(\t\x12\x17\n\x0fsample_fraction\x18\x04 \x01(\x01\x12\x16\n\x0e\x66uzzy_distance\x18\x05 \x01(\r\x12\x18\n\x10percentage_match\x18\x06 \x01(\r\"q\n$ListRecommendationClassifierResponse\x12I\n\x0b\x63lassifiers\x18\x01 \x03(\x0b\x32\x34.ai.h2o.featurestore.api.v1.RecommendationClassifier\"\xdd\x01\n\x18RecommendationClassifier\x12\x0c\n\x04name\x18\x01 \x01(\t\x12N\n\x05regex\x18\x02 \x01(\x0b\x32=.ai.h2o.featurestore.api.v1.RecommendationRegexMatchingPolicyH\x00\x12P\n\x06sample\x18\x03 \x01(\x0b\x32>.ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicyH\x00\x42\x11\n\x0fmatching_policy\"q\n%CreateRecommendationClassifierRequest\x12H\n\nclassifier\x18\x01 \x01(\x0b\x32\x34.ai.h2o.featurestore.api.v1.RecommendationClassifier\"q\n%UpdateRecommendationClassifierRequest\x12H\n\nclassifier\x18\x01 \x01(\x0b\x32\x34.ai.h2o.featurestore.api.v1.RecommendationClassifier\"@\n%DeleteRecommendationClassifierRequest\x12\x17\n\x0f\x63lassifier_name\x18\x01 \x01(\tB\x1d\x42\x1bRecommendationProtoApiProtob\x06proto3'
)




_RECOMMENDATIONREGEXMATCHINGPOLICY = _descriptor.Descriptor(
  name='RecommendationRegexMatchingPolicy',
  full_name='ai.h2o.featurestore.api.v1.RecommendationRegexMatchingPolicy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='regex', full_name='ai.h2o.featurestore.api.v1.RecommendationRegexMatchingPolicy.regex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='percentage_match', full_name='ai.h2o.featurestore.api.v1.RecommendationRegexMatchingPolicy.percentage_match', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=87,
  serialized_end=163,
)


_RECOMMENDATIONSAMPLEMATCHINGPOLICY = _descriptor.Descriptor(
  name='RecommendationSampleMatchingPolicy',
  full_name='ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_set_id', full_name='ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy.feature_set_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_set_major_version', full_name='ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy.feature_set_major_version', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='column_name', full_name='ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy.column_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_fraction', full_name='ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy.sample_fraction', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fuzzy_distance', full_name='ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy.fuzzy_distance', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='percentage_match', full_name='ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy.percentage_match', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=166,
  serialized_end=357,
)


_LISTRECOMMENDATIONCLASSIFIERRESPONSE = _descriptor.Descriptor(
  name='ListRecommendationClassifierResponse',
  full_name='ai.h2o.featurestore.api.v1.ListRecommendationClassifierResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='classifiers', full_name='ai.h2o.featurestore.api.v1.ListRecommendationClassifierResponse.classifiers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=359,
  serialized_end=472,
)


_RECOMMENDATIONCLASSIFIER = _descriptor.Descriptor(
  name='RecommendationClassifier',
  full_name='ai.h2o.featurestore.api.v1.RecommendationClassifier',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ai.h2o.featurestore.api.v1.RecommendationClassifier.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='regex', full_name='ai.h2o.featurestore.api.v1.RecommendationClassifier.regex', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample', full_name='ai.h2o.featurestore.api.v1.RecommendationClassifier.sample', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='matching_policy', full_name='ai.h2o.featurestore.api.v1.RecommendationClassifier.matching_policy',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=475,
  serialized_end=696,
)


_CREATERECOMMENDATIONCLASSIFIERREQUEST = _descriptor.Descriptor(
  name='CreateRecommendationClassifierRequest',
  full_name='ai.h2o.featurestore.api.v1.CreateRecommendationClassifierRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='classifier', full_name='ai.h2o.featurestore.api.v1.CreateRecommendationClassifierRequest.classifier', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=698,
  serialized_end=811,
)


_UPDATERECOMMENDATIONCLASSIFIERREQUEST = _descriptor.Descriptor(
  name='UpdateRecommendationClassifierRequest',
  full_name='ai.h2o.featurestore.api.v1.UpdateRecommendationClassifierRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='classifier', full_name='ai.h2o.featurestore.api.v1.UpdateRecommendationClassifierRequest.classifier', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=813,
  serialized_end=926,
)


_DELETERECOMMENDATIONCLASSIFIERREQUEST = _descriptor.Descriptor(
  name='DeleteRecommendationClassifierRequest',
  full_name='ai.h2o.featurestore.api.v1.DeleteRecommendationClassifierRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='classifier_name', full_name='ai.h2o.featurestore.api.v1.DeleteRecommendationClassifierRequest.classifier_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=928,
  serialized_end=992,
)

_LISTRECOMMENDATIONCLASSIFIERRESPONSE.fields_by_name['classifiers'].message_type = _RECOMMENDATIONCLASSIFIER
_RECOMMENDATIONCLASSIFIER.fields_by_name['regex'].message_type = _RECOMMENDATIONREGEXMATCHINGPOLICY
_RECOMMENDATIONCLASSIFIER.fields_by_name['sample'].message_type = _RECOMMENDATIONSAMPLEMATCHINGPOLICY
_RECOMMENDATIONCLASSIFIER.oneofs_by_name['matching_policy'].fields.append(
  _RECOMMENDATIONCLASSIFIER.fields_by_name['regex'])
_RECOMMENDATIONCLASSIFIER.fields_by_name['regex'].containing_oneof = _RECOMMENDATIONCLASSIFIER.oneofs_by_name['matching_policy']
_RECOMMENDATIONCLASSIFIER.oneofs_by_name['matching_policy'].fields.append(
  _RECOMMENDATIONCLASSIFIER.fields_by_name['sample'])
_RECOMMENDATIONCLASSIFIER.fields_by_name['sample'].containing_oneof = _RECOMMENDATIONCLASSIFIER.oneofs_by_name['matching_policy']
_CREATERECOMMENDATIONCLASSIFIERREQUEST.fields_by_name['classifier'].message_type = _RECOMMENDATIONCLASSIFIER
_UPDATERECOMMENDATIONCLASSIFIERREQUEST.fields_by_name['classifier'].message_type = _RECOMMENDATIONCLASSIFIER
DESCRIPTOR.message_types_by_name['RecommendationRegexMatchingPolicy'] = _RECOMMENDATIONREGEXMATCHINGPOLICY
DESCRIPTOR.message_types_by_name['RecommendationSampleMatchingPolicy'] = _RECOMMENDATIONSAMPLEMATCHINGPOLICY
DESCRIPTOR.message_types_by_name['ListRecommendationClassifierResponse'] = _LISTRECOMMENDATIONCLASSIFIERRESPONSE
DESCRIPTOR.message_types_by_name['RecommendationClassifier'] = _RECOMMENDATIONCLASSIFIER
DESCRIPTOR.message_types_by_name['CreateRecommendationClassifierRequest'] = _CREATERECOMMENDATIONCLASSIFIERREQUEST
DESCRIPTOR.message_types_by_name['UpdateRecommendationClassifierRequest'] = _UPDATERECOMMENDATIONCLASSIFIERREQUEST
DESCRIPTOR.message_types_by_name['DeleteRecommendationClassifierRequest'] = _DELETERECOMMENDATIONCLASSIFIERREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RecommendationRegexMatchingPolicy = _reflection.GeneratedProtocolMessageType('RecommendationRegexMatchingPolicy', (_message.Message,), {
  'DESCRIPTOR' : _RECOMMENDATIONREGEXMATCHINGPOLICY,
  '__module__' : 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.RecommendationRegexMatchingPolicy)
  })
_sym_db.RegisterMessage(RecommendationRegexMatchingPolicy)

RecommendationSampleMatchingPolicy = _reflection.GeneratedProtocolMessageType('RecommendationSampleMatchingPolicy', (_message.Message,), {
  'DESCRIPTOR' : _RECOMMENDATIONSAMPLEMATCHINGPOLICY,
  '__module__' : 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicy)
  })
_sym_db.RegisterMessage(RecommendationSampleMatchingPolicy)

ListRecommendationClassifierResponse = _reflection.GeneratedProtocolMessageType('ListRecommendationClassifierResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTRECOMMENDATIONCLASSIFIERRESPONSE,
  '__module__' : 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.ListRecommendationClassifierResponse)
  })
_sym_db.RegisterMessage(ListRecommendationClassifierResponse)

RecommendationClassifier = _reflection.GeneratedProtocolMessageType('RecommendationClassifier', (_message.Message,), {
  'DESCRIPTOR' : _RECOMMENDATIONCLASSIFIER,
  '__module__' : 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.RecommendationClassifier)
  })
_sym_db.RegisterMessage(RecommendationClassifier)

CreateRecommendationClassifierRequest = _reflection.GeneratedProtocolMessageType('CreateRecommendationClassifierRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATERECOMMENDATIONCLASSIFIERREQUEST,
  '__module__' : 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.CreateRecommendationClassifierRequest)
  })
_sym_db.RegisterMessage(CreateRecommendationClassifierRequest)

UpdateRecommendationClassifierRequest = _reflection.GeneratedProtocolMessageType('UpdateRecommendationClassifierRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATERECOMMENDATIONCLASSIFIERREQUEST,
  '__module__' : 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.UpdateRecommendationClassifierRequest)
  })
_sym_db.RegisterMessage(UpdateRecommendationClassifierRequest)

DeleteRecommendationClassifierRequest = _reflection.GeneratedProtocolMessageType('DeleteRecommendationClassifierRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETERECOMMENDATIONCLASSIFIERREQUEST,
  '__module__' : 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.DeleteRecommendationClassifierRequest)
  })
_sym_db.RegisterMessage(DeleteRecommendationClassifierRequest)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
